@@ -9,7 +9,7 @@
 ;;;;                onShoreD to support UncommonSQL front-end 
 ;;;; Date Started:  Feb 2002
 ;;;;
-;;;; $Id: db-interface.lisp 9890 2004-08-18 08:14:01Z kevin $
+;;;; $Id: db-interface.lisp 10612 2005-07-06 00:50:20Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2004 by Kevin M. Rosenberg
 ;;;; and Copyright (c) 1999-2001 by Pierre R. Mai, and onShoreD
@@ -452,4 +452,11 @@ of TYPE_NAME (keyword) PRECISION SCALE NULLABLE.")
   (unless (is-database-open database)
     (signal-closed-database-error database)))
 
+(defvar *foreign-library-search-paths* nil
+  "A list of pathnames denoting directories where CLSQL will look
+for foreign libraries \(in addition to the default places).")
 
+(defun push-library-path (path)
+  "Adds the pathspec PATH \(which should denote a directory) to
+the list *FOREIGN-LIBRARY-SEARCH-PATHS*."
+  (push path *foreign-library-search-paths*))
\ No newline at end of file
