@@ -1,7 +1,7 @@
 ;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
 ;;;; *************************************************************************
 ;;;;
-;;;; $Id: metaclasses.lisp 10190 2004-12-07 06:10:37Z kevin $
+;;;; $Id: metaclasses.lisp 10293 2005-01-29 00:54:17Z kevin $
 ;;;;
 ;;;; CLSQL metaclass for standard-db-objects created in the OODDL. 
 ;;;;
@@ -78,7 +78,7 @@
 	((typep arg 'sql-ident)
 	 (slot-value arg 'name))
 	((stringp arg)
-	 (intern (symbol-name-default-case arg)))))
+	 (intern arg))))
 
 (defun column-name-from-arg (arg)
   (cond ((symbolp arg)
@@ -309,7 +309,7 @@ column definition in the database.")
 (defun parse-db-info (db-info-list)
   (destructuring-bind
 	(&key join-class home-key key-join foreign-key (delete-rule nil)
-	      (target-slot nil) (retrieval :deferred) (set nil))
+	      (target-slot nil) (retrieval :deferred) (set t))
       db-info-list
     (let ((ih (make-hash-table :size 6)))
       (if join-class
