@@ -1,7 +1,7 @@
 ;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
 ;;;; *************************************************************************
 ;;;;
-;;;; $Id: oodml.lisp 10190 2004-12-07 06:10:37Z kevin $
+;;;; $Id: oodml.lisp 10540 2005-05-17 19:08:07Z kevin $
 ;;;;
 ;;;; The CLSQL Object Oriented Data Manipulation Language (OODML).
 ;;;;
@@ -103,7 +103,7 @@
           ((typep slot-reader 'string)
            (setf (slot-value instance slot-name)
                  (format nil slot-reader value)))
-          ((typep slot-reader 'function)
+          ((typep slot-reader '(or symbol function))
            (setf (slot-value instance slot-name)
                  (apply slot-reader (list value))))
           (t
@@ -120,7 +120,7 @@
            nil)
           ((typep slot-reader 'string)
            (format nil slot-reader value))
-          ((typep slot-reader 'function)
+          ((typep slot-reader '(or symbol function))
            (apply slot-reader (list value)))
           (t
            (error "Slot reader is of an unusual type.")))))
@@ -130,7 +130,7 @@
 	(dbtype (specified-type slotdef)))
     (typecase dbwriter
       (string (format nil dbwriter val))
-      (function (apply dbwriter (list val)))
+      ((and (or symbol function) (not null)) (apply dbwriter (list val)))
       (t
        (database-output-sql-as-type
 	(typecase dbtype
@@ -211,9 +211,8 @@
            (error "Unable to update records"))))
   (values))
 
-(defmethod update-records-from-instance ((obj standard-db-object)
-                                         &key (database *default-database*))
-  (let ((database (or (view-database obj) database)))
+(defmethod update-records-from-instance ((obj standard-db-object) &key database)
+  (let ((database (or database (view-database obj) *default-database*)))
     (labels ((slot-storedp (slot)
 	       (and (member (view-class-slot-db-kind slot) '(:base :key))
 		    (slot-boundp obj (slot-definition-name slot))))
@@ -248,6 +247,7 @@
     (if vd
 	(let ((qualifier (key-qualifier-for-instance instance :database vd)))
 	  (delete-records :from vt :where qualifier :database vd)
+	  (setf (record-caches vd) nil)
 	  (setf (slot-value instance 'view-database) nil)
           (values))
 	(signal-no-database-error vd))))
@@ -261,8 +261,9 @@
          (sels (generate-selection-list view-class))
          (res (apply #'select (append (mapcar #'cdr sels)
                                       (list :from  view-table
-                                            :where view-qual)
-				      (list :result-types nil)))))
+                                            :where view-qual
+					    :result-types nil
+					    :database vd)))))
     (when res
       (get-slot-values-from-view instance (mapcar #'car sels) (car res)))))
 
@@ -642,7 +643,8 @@
 				:attribute (gethash :home-key tdbi) 
 				:table jc-view-table))
 			  :where jq
-			  :result-types :auto)))
+			  :result-types :auto
+			  :database (view-database object))))
 	   (mapcar #'(lambda (i)
 		       (let* ((instance (car i))
 			      (jcc (make-instance jc :view-database (view-database instance))))
@@ -667,7 +669,8 @@
 		   (list instance jcc)))
 	     (select (sql-expression :attribute (gethash :foreign-key tdbi) :table jc-view-table)
 		     :from (sql-expression :table jc-view-table)
-		     :where jq)))))))
+		     :where jq
+		     :database (view-database object))))))))
 
 
 ;;; Remote Joins
