@@ -5,7 +5,7 @@
 ;;;; Name:          package.lisp
 ;;;; Purpose:       Package definition for SQL interface
 ;;;;
-;;;; $Id: package.lisp 10074 2004-10-01 09:19:18Z kevin $
+;;;; $Id: package.lisp 10612 2005-07-06 00:50:20Z kevin $
 ;;;; 
 ;;;; This file is part of CLSQL.
 ;;;;
@@ -110,6 +110,7 @@
      #:database-type-load-foreign
      #:database-connect
      #:database-disconnect
+     #:database-reconnect
      #:database-query
      #:database-execute-command
      #:database-create-sequence
@@ -225,6 +226,10 @@
      ;; Shared exports for re-export by CLSQL package. 
      #1=(
 
+         ;; foreign library loading
+         #:*foreign-library-search-paths*
+         #:push-library-path
+
 	 ;; Condition system (conditions.lisp) 
 	 #:sql-user-error
 	 #:sql-database-error
