@@ -7,7 +7,7 @@
 ;;;; Programmer:   Kevin M. Rosenberg
 ;;;; Date Started: Mar 2002
 ;;;;
-;;;; $Id: utils.lisp 10078 2004-10-01 17:24:52Z kevin $
+;;;; $Id: utils.lisp 10284 2005-01-27 05:56:00Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2004 by Kevin M. Rosenberg
 ;;;;
@@ -52,10 +52,9 @@
 
 (defun sql-escape (identifier)
   "Change hyphens to underscores, ensure string"
-  (let* ((unescaped (etypecase identifier
-                      (symbol (symbol-name identifier))
-                      (string identifier)))
-         (escaped (make-string (length unescaped))))
+  (let ((unescaped (etypecase identifier
+		     (symbol (symbol-name identifier))
+		     (string identifier))))
     (substitute #\_ #\- unescaped)))
 
 (defmacro without-interrupts (&body body)
