@@ -3,7 +3,7 @@
 ;;;; File:    test-init.lisp
 ;;;; Authors: Marcus Pearce <m.t.pearce@city.ac.uk>, Kevin Rosenberg
 ;;;; Created: 30/03/2004
-;;;; Updated: $Id: test-init.lisp 10128 2004-10-31 23:59:26Z kevin $
+;;;; Updated: $Id: test-init.lisp 10281 2005-01-26 18:59:10Z kevin $
 ;;;;
 ;;;; Initialisation utilities for running regression tests on CLSQL. 
 ;;;;
@@ -259,7 +259,7 @@
 				   :height (1+ (random 1.00))
 				   :bd-utime *test-start-utime*
 				   :birthday now-time
-				   :first-name "Vladamir"
+				   :first-name "Vladimir"
 				   :last-name "Lenin"
 				   :email "lenin@soviet.org"
 				   :companyid 1)
@@ -366,7 +366,7 @@
 				    :height (1+ (random 1.00))
 				    :bd-utime *test-start-utime*
 				    :birthday now-time
-				    :first-name "Vladamir"
+				    :first-name "Vladimir"
 				    :last-name "Putin"
 				    :email "putin@soviet.org"
 				    :managerid 1
